<?php

namespace Drupal\workbench_approver_access\Access;

use Drupal\Core\Access\AccessResult;
use Drupal\Core\Routing\Access\AccessInterface;
use Drupal\Core\Routing\RouteMatchInterface;
use Drupal\Core\Session\AccountInterface;
use Drupal\user\EntityOwnerInterface;
use Drupal\workbench_approver\Access\ApproveLatestRevisionCheck;
use Symfony\Component\Routing\Route;

/**
 * Grants access to the latest revision tab.
 */
class AccessApproveLatestRevisionCheck extends ApproveLatestRevisionCheck implements AccessInterface {

  /**
   * {@inheritdoc}
   */
  public function access(Route $route, RouteMatchInterface $route_match, AccountInterface $account) {
    // @todo Should we just call parent::access() here?
    // This tab should not show up unless there's a reason to show it.
    $entity = $this->loadEntity($route, $route_match);
    if ($this->moderationInfo->hasPendingRevision($entity) && $this->moderationInfo->isModeratedEntity($entity)) {
      // The entity hook is the direct method for getting access.
      // Update here is a sufficient check to learn if the entity
      // is within scope of the user.
      $access_result = workbench_access_entity_access($entity, 'update', $account);
      // A neutral result from the above grants access because workbench_access
      // only returns neutral or forbidden.
      if ($access_result->isNeutral()) {
        // Check the global permissions first.
        $access_result = AccessResult::allowedIfHasPermissions($account, [
          'view latest version',
          'view any unpublished content',
        ]);
        if (!$access_result->isAllowed()) {
          // Check entity owner access.
          $owner_access = AccessResult::allowedIfHasPermissions($account, [
            'view latest version',
            'view own unpublished content',
          ]);
          $owner_access = $owner_access->andIf((AccessResult::allowedIf($entity instanceof EntityOwnerInterface && ($entity->getOwnerId() == $account->id()))));
          $access_result = $access_result->orIf($owner_access);
          // This is our new check; it expands the allowed list.
          if (!$access_result->isAllowed() && $this->checkAssignments($entity, $account)) {
            $access_result = AccessResult::allowedIfHasPermission($account, 'use workbench approver');
            // @todo Cache by $approver entity?
            $access_result->addCacheableDependency($account);
          }
        }
      }

      return $access_result->addCacheableDependency($entity);
    }

    return AccessResult::forbidden('No pending revision for moderated entity.')->addCacheableDependency($entity);
  }

}
