<?php

namespace Drupal\workbench_approver_access\Access;

use Drupal\Core\Access\AccessResult;
use Drupal\Core\Routing\Access\AccessInterface;
use Drupal\Core\Routing\RouteMatchInterface;
use Drupal\Core\Session\AccountInterface;
use Drupal\workbench_approver\Access\ViewApprovalsCheck;
use Symfony\Component\Routing\Route;

/**
 * Grants access to the Approve tab.
 */
class AccessViewApprovalsCheck extends ViewApprovalsCheck implements AccessInterface {

  /**
   * {@inheritdoc}
   */
  public function access(Route $route, RouteMatchInterface $route_match, AccountInterface $account) {
    // This tab should not show up unless there's a reason to show it.
    $entity = $this->loadEntity($route, $route_match);
    if ($this->moderationInfo->isModeratedEntity($entity)) {
      // The entity hook is the direct method for getting access.
      // Update here is a sufficient check to learn if the entity
      // is within scope of the user.
      $access_result = workbench_access_entity_access($entity, 'update', $account);
      // A neutral result from the above grants access because workbench_access
      // only returns neutral or forbidden.
      if ($access_result->isNeutral()) {
        $access_result = AccessResult::allowedIfHasPermission($account, 'view workbench approver information');
      }

      return $access_result->addCacheableDependency($entity);
    }

    return AccessResult::forbidden('No pending revision for moderated entity.')->addCacheableDependency($entity);
  }

}
